/* ------------------------------------------------------------
author: "Albert Graef"
name: "chimes"
version: "2.0"
Code generated with Faust 2.41.1 (https://faust.grame.fr)
Compilation options: -a puredata.cpp -lang cpp -es 1 -mcd 16 -single -ftz 0
------------------------------------------------------------ */

#ifndef  __mydsp_H__
#define  __mydsp_H__

/************************************************************************
 ************************************************************************
    FAUST Architecture File
    Copyright (C) 2006-2011 Albert Graef <Dr.Graef@t-online.de>
    ---------------------------------------------------------------------
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as 
    published by the Free Software Foundation; either version 2.1 of the 
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with the GNU C Library; if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
    02111-1307 USA. 
 ************************************************************************
 ************************************************************************/

/* Pd architecture file, written by Albert Graef <Dr.Graef@t-online.de>.
   This was derived from minimal.cpp included in the Faust distribution.
   Please note that this is to be compiled as a shared library, which is
   then loaded dynamically by Pd as an external. */

#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "faust/misc.h"
#include "faust/gui/UI.h"
#include "faust/gui/meta.h"
#include "faust/dsp/dsp.h"

/******************************************************************************
*******************************************************************************

							       VECTOR INTRINSICS

*******************************************************************************
*******************************************************************************/


/***************************************************************************
   Pd UI interface
 ***************************************************************************/

enum ui_elem_type_t {
  UI_BUTTON, UI_CHECK_BUTTON,
  UI_V_SLIDER, UI_H_SLIDER, UI_NUM_ENTRY,
  UI_V_BARGRAPH, UI_H_BARGRAPH,
  UI_END_GROUP, UI_V_GROUP, UI_H_GROUP, UI_T_GROUP
};

struct ui_elem_t {
  ui_elem_type_t type;
  char *label;
  float *zone;
  float init, min, max, step;
};

class PdUI : public UI
{
public:
  const char *name;
  int nelems, level;
  ui_elem_t *elems;
		
  PdUI();
  PdUI(const char *nm, const char *s);
  virtual ~PdUI();

protected:
  std::string path;
  void add_elem(ui_elem_type_t type, const char *label = NULL);
  void add_elem(ui_elem_type_t type, const char *label, float *zone);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float init, float min, float max, float step);
  void add_elem(ui_elem_type_t type, const char *label, float *zone,
		float min, float max);

public:
  virtual void addButton(const char* label, float* zone);
  virtual void addCheckButton(const char* label, float* zone);
  virtual void addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step);
  virtual void addNumEntry(const char* label, float* zone, float init, float min, float max, float step);

  virtual void addHorizontalBargraph(const char* label, float* zone, float min, float max);
  virtual void addVerticalBargraph(const char* label, float* zone, float min, float max);
    
  virtual void addSoundfile(const char* label, const char* filename, Soundfile** sf_zone) {}
  
  virtual void openTabBox(const char* label);
  virtual void openHorizontalBox(const char* label);
  virtual void openVerticalBox(const char* label);
  virtual void closeBox();
	
  virtual void run();
};

static std::string mangle(const char *name, int level, const char *s)
{
  const char *s0 = s;
  std::string t = "";
  if (!s) return t;
  // Get rid of bogus "0x00" labels in recent Faust revisions. Also, for
  // backward compatibility with old Faust versions, make sure that default
  // toplevel groups and explicit toplevel groups with an empty label are
  // treated alike (these both return "0x00" labels in the latest Faust, but
  // would be treated inconsistently in earlier versions).
  if (!*s || strcmp(s, "0x00") == 0) {
    if (level == 0)
      // toplevel group with empty label, map to dsp name
      s = name;
    else
      // empty label
      s = "";
  }
  while (*s)
    if (isalnum(*s))
      t += *(s++);
    else {
      const char *s1 = s;
      while (*s && !isalnum(*s)) ++s;
      if (s1 != s0 && *s) t += "-";
    }
  return t;
}

static std::string normpath(std::string path)
{
  path = std::string("/")+path;
  int pos = path.find("//");
  while (pos >= 0) {
    path.erase(pos, 1);
    pos = path.find("//");
  }
  size_t len = path.length();
  if (len > 1 && path[len-1] == '/')
    path.erase(len-1, 1);
  return path;
}

static std::string pathcat(std::string path, std::string label)
{
  if (path.empty())
    return normpath(label);
  else if (label.empty())
    return normpath(path);
  else
    return normpath(path+"/"+label);
}

PdUI::PdUI()
{
  nelems = level = 0;
  elems = NULL;
  name = "";
  path = "";
}

PdUI::PdUI(const char *nm, const char *s)
{
  nelems = level = 0;
  elems = NULL;
  name = nm?nm:"";
  path = s?s:"";
}

PdUI::~PdUI()
{
  if (elems) {
    for (int i = 0; i < nelems; i++)
      if (elems[i].label)
	free(elems[i].label);
    free(elems);
  }
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = NULL;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 0.0;
  elems[nelems].step = 0.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = 0.0;
  elems[nelems].max = 1.0;
  elems[nelems].step = 1.0;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float init, float min, float max, float step)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = init;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = step;
  nelems++;
}

inline void PdUI::add_elem(ui_elem_type_t type, const char *label, float *zone,
			  float min, float max)
{
  ui_elem_t *elems1 = (ui_elem_t*)realloc(elems, (nelems+1)*sizeof(ui_elem_t));
  if (elems1)
    elems = elems1;
  else
    return;
  std::string s = pathcat(path, mangle(name, level, label));
  elems[nelems].type = type;
  elems[nelems].label = strdup(s.c_str());
  elems[nelems].zone = zone;
  elems[nelems].init = 0.0;
  elems[nelems].min = min;
  elems[nelems].max = max;
  elems[nelems].step = 0.0;
  nelems++;
}

void PdUI::addButton(const char* label, float* zone)
{ add_elem(UI_BUTTON, label, zone); }
void PdUI::addCheckButton(const char* label, float* zone)
{ add_elem(UI_CHECK_BUTTON, label, zone); }
void PdUI::addVerticalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_V_SLIDER, label, zone, init, min, max, step); }
void PdUI::addHorizontalSlider(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_H_SLIDER, label, zone, init, min, max, step); }
void PdUI::addNumEntry(const char* label, float* zone, float init, float min, float max, float step)
{ add_elem(UI_NUM_ENTRY, label, zone, init, min, max, step); }

void PdUI::addHorizontalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_H_BARGRAPH, label, zone, min, max); }
void PdUI::addVerticalBargraph(const char* label, float* zone, float min, float max)
{ add_elem(UI_V_BARGRAPH, label, zone, min, max); }

void PdUI::openTabBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::openHorizontalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::openVerticalBox(const char* label)
{
  if (!path.empty()) path += "/";
  path += mangle(name, level, label);
  level++;
}
void PdUI::closeBox()
{
  int pos = path.rfind("/");
  if (pos < 0) pos = 0;
  path.erase(pos);
  level--;
}

void PdUI::run() {}

/******************************************************************************
*******************************************************************************

			    FAUST DSP

*******************************************************************************
*******************************************************************************/

//----------------------------------------------------------------------------
//  FAUST generated signal processor
//----------------------------------------------------------------------------
		
#ifndef FAUSTFLOAT
#define FAUSTFLOAT float
#endif 

#include <algorithm>
#include <cmath>
#include <cstdint>
#include <math.h>

#ifndef FAUSTCLASS 
#define FAUSTCLASS mydsp
#endif

#ifdef __APPLE__ 
#define exp10f __exp10f
#define exp10 __exp10
#endif

#if defined(_WIN32)
#define RESTRICT __restrict
#else
#define RESTRICT __restrict__
#endif

static float mydsp_faustpower2_f(float value) {
	return value * value;
}

class mydsp : public dsp {
	
 private:
	
	int fSampleRate;
	float fConst0;
	float fConst1;
	FAUSTFLOAT fHslider0;
	FAUSTFLOAT fHslider1;
	FAUSTFLOAT fEntry0;
	FAUSTFLOAT fEntry1;
	FAUSTFLOAT fEntry2;
	float fConst2;
	float fConst3;
	FAUSTFLOAT fEntry3;
	FAUSTFLOAT fEntry4;
	FAUSTFLOAT fButton0;
	int iRec1[2];
	FAUSTFLOAT fEntry5;
	float fRec2[2];
	int iRec3[2];
	float fVec0[2];
	float fRec0[3];
	int iRec4[2];
	FAUSTFLOAT fEntry6;
	float fRec5[2];
	FAUSTFLOAT fEntry7;
	FAUSTFLOAT fEntry8;
	FAUSTFLOAT fEntry9;
	float fRec6[3];
	int iRec7[2];
	FAUSTFLOAT fEntry10;
	float fRec8[2];
	FAUSTFLOAT fEntry11;
	FAUSTFLOAT fEntry12;
	FAUSTFLOAT fEntry13;
	float fRec9[3];
	int iRec10[2];
	FAUSTFLOAT fEntry14;
	float fRec11[2];
	FAUSTFLOAT fEntry15;
	FAUSTFLOAT fEntry16;
	FAUSTFLOAT fEntry17;
	float fRec12[3];
	int iRec13[2];
	FAUSTFLOAT fEntry18;
	float fRec14[2];
	FAUSTFLOAT fEntry19;
	FAUSTFLOAT fEntry20;
	FAUSTFLOAT fEntry21;
	float fRec15[3];
	int iRec16[2];
	FAUSTFLOAT fEntry22;
	float fRec17[2];
	
 public:
	
	void metadata(Meta* m) { 
		m->declare("author", "Albert Graef");
		m->declare("compile_options", "-a puredata.cpp -lang cpp -es 1 -mcd 16 -single -ftz 0");
		m->declare("description", "chimes synth using a bank of resonz filters");
		m->declare("filename", "chimes.syn");
		m->declare("math.lib/author", "GRAME");
		m->declare("math.lib/copyright", "GRAME");
		m->declare("math.lib/deprecated", "This library is deprecated and is not maintained anymore. It will be removed in August 2017.");
		m->declare("math.lib/license", "LGPL with exception");
		m->declare("math.lib/name", "Math Library");
		m->declare("math.lib/version", "1.0");
		m->declare("music.lib/author", "GRAME");
		m->declare("music.lib/copyright", "GRAME");
		m->declare("music.lib/deprecated", "This library is deprecated and is not maintained anymore. It will be removed in August 2017.");
		m->declare("music.lib/license", "LGPL with exception");
		m->declare("music.lib/name", "Music Library");
		m->declare("music.lib/version", "1.0");
		m->declare("name", "chimes");
		m->declare("version", "2.0");
	}

	virtual int getNumInputs() {
		return 0;
	}
	virtual int getNumOutputs() {
		return 2;
	}
	
	static void classInit(int sample_rate) {
	}
	
	virtual void instanceConstants(int sample_rate) {
		fSampleRate = sample_rate;
		fConst0 = std::min<float>(192000.0f, std::max<float>(1.0f, float(fSampleRate)));
		fConst1 = 0.5f * fConst0;
		fConst2 = 3.14159274f / fConst0;
		fConst3 = 6.28318548f / fConst0;
	}
	
	virtual void instanceResetUserInterface() {
		fHslider0 = FAUSTFLOAT(0.5f);
		fHslider1 = FAUSTFLOAT(0.5f);
		fEntry0 = FAUSTFLOAT(440.0f);
		fEntry1 = FAUSTFLOAT(0.16700000000000001f);
		fEntry2 = FAUSTFLOAT(0.002f);
		fEntry3 = FAUSTFLOAT(1.0f);
		fEntry4 = FAUSTFLOAT(1.0f);
		fButton0 = FAUSTFLOAT(0.0f);
		fEntry5 = FAUSTFLOAT(0.01f);
		fEntry6 = FAUSTFLOAT(3.6930000000000001f);
		fEntry7 = FAUSTFLOAT(0.083000000000000004f);
		fEntry8 = FAUSTFLOAT(0.002f);
		fEntry9 = FAUSTFLOAT(3.0070000000000001f);
		fEntry10 = FAUSTFLOAT(2.2480000000000002f);
		fEntry11 = FAUSTFLOAT(0.086999999999999994f);
		fEntry12 = FAUSTFLOAT(0.002f);
		fEntry13 = FAUSTFLOAT(4.968f);
		fEntry14 = FAUSTFLOAT(2.8279999999999998f);
		fEntry15 = FAUSTFLOAT(0.052999999999999999f);
		fEntry16 = FAUSTFLOAT(0.002f);
		fEntry17 = FAUSTFLOAT(8.9939999999999998f);
		fEntry18 = FAUSTFLOAT(3.3639999999999999f);
		fEntry19 = FAUSTFLOAT(0.052999999999999999f);
		fEntry20 = FAUSTFLOAT(0.002f);
		fEntry21 = FAUSTFLOAT(12.006f);
		fEntry22 = FAUSTFLOAT(2.488f);
	}
	
	virtual void instanceClear() {
		for (int l0 = 0; l0 < 2; l0 = l0 + 1) {
			iRec1[l0] = 0;
		}
		for (int l1 = 0; l1 < 2; l1 = l1 + 1) {
			fRec2[l1] = 0.0f;
		}
		for (int l2 = 0; l2 < 2; l2 = l2 + 1) {
			iRec3[l2] = 0;
		}
		for (int l3 = 0; l3 < 2; l3 = l3 + 1) {
			fVec0[l3] = 0.0f;
		}
		for (int l4 = 0; l4 < 3; l4 = l4 + 1) {
			fRec0[l4] = 0.0f;
		}
		for (int l5 = 0; l5 < 2; l5 = l5 + 1) {
			iRec4[l5] = 0;
		}
		for (int l6 = 0; l6 < 2; l6 = l6 + 1) {
			fRec5[l6] = 0.0f;
		}
		for (int l7 = 0; l7 < 3; l7 = l7 + 1) {
			fRec6[l7] = 0.0f;
		}
		for (int l8 = 0; l8 < 2; l8 = l8 + 1) {
			iRec7[l8] = 0;
		}
		for (int l9 = 0; l9 < 2; l9 = l9 + 1) {
			fRec8[l9] = 0.0f;
		}
		for (int l10 = 0; l10 < 3; l10 = l10 + 1) {
			fRec9[l10] = 0.0f;
		}
		for (int l11 = 0; l11 < 2; l11 = l11 + 1) {
			iRec10[l11] = 0;
		}
		for (int l12 = 0; l12 < 2; l12 = l12 + 1) {
			fRec11[l12] = 0.0f;
		}
		for (int l13 = 0; l13 < 3; l13 = l13 + 1) {
			fRec12[l13] = 0.0f;
		}
		for (int l14 = 0; l14 < 2; l14 = l14 + 1) {
			iRec13[l14] = 0;
		}
		for (int l15 = 0; l15 < 2; l15 = l15 + 1) {
			fRec14[l15] = 0.0f;
		}
		for (int l16 = 0; l16 < 3; l16 = l16 + 1) {
			fRec15[l16] = 0.0f;
		}
		for (int l17 = 0; l17 < 2; l17 = l17 + 1) {
			iRec16[l17] = 0;
		}
		for (int l18 = 0; l18 < 2; l18 = l18 + 1) {
			fRec17[l18] = 0.0f;
		}
	}
	
	virtual void init(int sample_rate) {
		classInit(sample_rate);
		instanceInit(sample_rate);
	}
	virtual void instanceInit(int sample_rate) {
		instanceConstants(sample_rate);
		instanceResetUserInterface();
		instanceClear();
	}
	
	virtual mydsp* clone() {
		return new mydsp();
	}
	
	virtual int getSampleRate() {
		return fSampleRate;
	}
	
	virtual void buildUserInterface(UI* ui_interface) {
		ui_interface->openVerticalBox("chimes");
		ui_interface->openHorizontalBox("1-excitator");
		ui_interface->addNumEntry("decay", &fEntry5, FAUSTFLOAT(0.00999999978f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("2-resonator-0");
		ui_interface->addNumEntry("amplitude0", &fEntry1, FAUSTFLOAT(0.166999996f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("decay0", &fEntry6, FAUSTFLOAT(3.69300008f), FAUSTFLOAT(0.0f), FAUSTFLOAT(10.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("harmonic0", &fEntry3, FAUSTFLOAT(1.0f), FAUSTFLOAT(0.0f), FAUSTFLOAT(50.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("rq0", &fEntry2, FAUSTFLOAT(0.00200000009f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(9.99999975e-05f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("2-resonator-1");
		ui_interface->addNumEntry("amplitude1", &fEntry7, FAUSTFLOAT(0.0829999968f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("decay1", &fEntry10, FAUSTFLOAT(2.24799991f), FAUSTFLOAT(0.0f), FAUSTFLOAT(10.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("harmonic1", &fEntry9, FAUSTFLOAT(3.00699997f), FAUSTFLOAT(0.0f), FAUSTFLOAT(50.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("rq1", &fEntry8, FAUSTFLOAT(0.00200000009f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(9.99999975e-05f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("2-resonator-2");
		ui_interface->addNumEntry("amplitude2", &fEntry11, FAUSTFLOAT(0.0869999975f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("decay2", &fEntry14, FAUSTFLOAT(2.82800007f), FAUSTFLOAT(0.0f), FAUSTFLOAT(10.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("harmonic2", &fEntry13, FAUSTFLOAT(4.96799994f), FAUSTFLOAT(0.0f), FAUSTFLOAT(50.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("rq2", &fEntry12, FAUSTFLOAT(0.00200000009f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(9.99999975e-05f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("2-resonator-3");
		ui_interface->addNumEntry("amplitude3", &fEntry15, FAUSTFLOAT(0.0529999994f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("decay3", &fEntry18, FAUSTFLOAT(3.36400008f), FAUSTFLOAT(0.0f), FAUSTFLOAT(10.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("harmonic3", &fEntry17, FAUSTFLOAT(8.99400043f), FAUSTFLOAT(0.0f), FAUSTFLOAT(50.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("rq3", &fEntry16, FAUSTFLOAT(0.00200000009f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(9.99999975e-05f));
		ui_interface->closeBox();
		ui_interface->openHorizontalBox("2-resonator-4");
		ui_interface->addNumEntry("amplitude4", &fEntry19, FAUSTFLOAT(0.0529999994f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("decay4", &fEntry22, FAUSTFLOAT(2.48799992f), FAUSTFLOAT(0.0f), FAUSTFLOAT(10.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("harmonic4", &fEntry21, FAUSTFLOAT(12.0059996f), FAUSTFLOAT(0.0f), FAUSTFLOAT(50.0f), FAUSTFLOAT(0.00100000005f));
		ui_interface->addNumEntry("rq4", &fEntry20, FAUSTFLOAT(0.00200000009f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(9.99999975e-05f));
		ui_interface->closeBox();
		ui_interface->openVerticalBox("3-master");
		ui_interface->addHorizontalSlider("pan", &fHslider1, FAUSTFLOAT(0.5f), FAUSTFLOAT(0.0f), FAUSTFLOAT(1.0f), FAUSTFLOAT(0.00999999978f));
		ui_interface->addHorizontalSlider("vol", &fHslider0, FAUSTFLOAT(0.5f), FAUSTFLOAT(0.0f), FAUSTFLOAT(10.0f), FAUSTFLOAT(0.00999999978f));
		ui_interface->closeBox();
		ui_interface->addNumEntry("freq", &fEntry0, FAUSTFLOAT(440.0f), FAUSTFLOAT(20.0f), FAUSTFLOAT(20000.0f), FAUSTFLOAT(1.0f));
		ui_interface->addNumEntry("gain", &fEntry4, FAUSTFLOAT(1.0f), FAUSTFLOAT(0.0f), FAUSTFLOAT(10.0f), FAUSTFLOAT(0.00999999978f));
		ui_interface->addButton("gate", &fButton0);
		ui_interface->closeBox();
	}
	
	virtual void compute(int count, FAUSTFLOAT** RESTRICT inputs, FAUSTFLOAT** RESTRICT outputs) {
		FAUSTFLOAT* output0 = outputs[0];
		FAUSTFLOAT* output1 = outputs[1];
		float fSlow0 = float(fHslider0);
		float fSlow1 = float(fHslider1);
		float fSlow2 = float(fEntry0);
		float fSlow3 = fConst1 * (fSlow0 * (1.0f - fSlow1)) / fSlow2;
		float fSlow4 = float(fEntry2);
		float fSlow5 = float(fEntry1) / fSlow4;
		float fSlow6 = mydsp_faustpower2_f(1.0f - fConst2 * fSlow2 * fSlow4);
		float fSlow7 = std::cos(fConst3 * fSlow2 * float(fEntry3));
		float fSlow8 = fSlow6 + 1.0f;
		float fSlow9 = 4.0f * fSlow7 / fSlow8;
		float fSlow10 = (1.0f - fSlow6) * std::sqrt(1.0f - 4.0f * (fSlow6 * mydsp_faustpower2_f(fSlow7)) / mydsp_faustpower2_f(fSlow8));
		float fSlow11 = 4.65661287e-10f * float(fEntry4);
		float fSlow12 = float(fButton0);
		int iSlow13 = fSlow12 > 0.0f;
		float fSlow14 = float(fEntry5);
		float fSlow15 = 1.0f - std::pow(0.00100000005f, 1.0f / (fConst0 * fSlow14 + float(fSlow14 == 0.0f)));
		int iSlow16 = fSlow12 <= 0.0f;
		float fSlow17 = float(fEntry6);
		float fSlow18 = 1.0f - std::pow(0.00100000005f, 1.0f / (fConst0 * fSlow17 + float(fSlow17 == 0.0f)));
		float fSlow19 = float(fEntry8);
		float fSlow20 = float(fEntry7) / fSlow19;
		float fSlow21 = mydsp_faustpower2_f(1.0f - fConst2 * fSlow2 * fSlow19);
		float fSlow22 = std::cos(fConst3 * fSlow2 * float(fEntry9));
		float fSlow23 = fSlow21 + 1.0f;
		float fSlow24 = 4.0f * fSlow22 / fSlow23;
		float fSlow25 = (1.0f - fSlow21) * std::sqrt(1.0f - 4.0f * (fSlow21 * mydsp_faustpower2_f(fSlow22)) / mydsp_faustpower2_f(fSlow23));
		float fSlow26 = float(fEntry10);
		float fSlow27 = 1.0f - std::pow(0.00100000005f, 1.0f / (fConst0 * fSlow26 + float(fSlow26 == 0.0f)));
		float fSlow28 = float(fEntry12);
		float fSlow29 = float(fEntry11) / fSlow28;
		float fSlow30 = mydsp_faustpower2_f(1.0f - fConst2 * fSlow2 * fSlow28);
		float fSlow31 = std::cos(fConst3 * fSlow2 * float(fEntry13));
		float fSlow32 = fSlow30 + 1.0f;
		float fSlow33 = 4.0f * fSlow31 / fSlow32;
		float fSlow34 = (1.0f - fSlow30) * std::sqrt(1.0f - 4.0f * (fSlow30 * mydsp_faustpower2_f(fSlow31)) / mydsp_faustpower2_f(fSlow32));
		float fSlow35 = float(fEntry14);
		float fSlow36 = 1.0f - std::pow(0.00100000005f, 1.0f / (fConst0 * fSlow35 + float(fSlow35 == 0.0f)));
		float fSlow37 = float(fEntry16);
		float fSlow38 = float(fEntry15) / fSlow37;
		float fSlow39 = mydsp_faustpower2_f(1.0f - fConst2 * fSlow2 * fSlow37);
		float fSlow40 = std::cos(fConst3 * fSlow2 * float(fEntry17));
		float fSlow41 = fSlow39 + 1.0f;
		float fSlow42 = 4.0f * fSlow40 / fSlow41;
		float fSlow43 = (1.0f - fSlow39) * std::sqrt(1.0f - 4.0f * (fSlow39 * mydsp_faustpower2_f(fSlow40)) / mydsp_faustpower2_f(fSlow41));
		float fSlow44 = float(fEntry18);
		float fSlow45 = 1.0f - std::pow(0.00100000005f, 1.0f / (fConst0 * fSlow44 + float(fSlow44 == 0.0f)));
		float fSlow46 = float(fEntry20);
		float fSlow47 = float(fEntry19) / fSlow46;
		float fSlow48 = mydsp_faustpower2_f(1.0f - fConst2 * fSlow2 * fSlow46);
		float fSlow49 = std::cos(fConst3 * fSlow2 * float(fEntry21));
		float fSlow50 = fSlow48 + 1.0f;
		float fSlow51 = 4.0f * fSlow49 / fSlow50;
		float fSlow52 = (1.0f - fSlow48) * std::sqrt(1.0f - 4.0f * (fSlow48 * mydsp_faustpower2_f(fSlow49)) / mydsp_faustpower2_f(fSlow50));
		float fSlow53 = float(fEntry22);
		float fSlow54 = 1.0f - std::pow(0.00100000005f, 1.0f / (fConst0 * fSlow53 + float(fSlow53 == 0.0f)));
		float fSlow55 = fConst1 * (fSlow0 * fSlow1) / fSlow2;
		for (int i0 = 0; i0 < count; i0 = i0 + 1) {
			iRec1[0] = iSlow13 & (iRec1[1] | (fRec2[1] >= 1.0f));
			int iTemp0 = fRec2[1] > 0.0f;
			fRec2[0] = (float(((iRec1[1] == 0) & iSlow13) & (fRec2[1] < 1.0f)) + fRec2[1] * (1.0f - fSlow15 * float(iRec1[1] & iTemp0))) * float(((iSlow16 & iTemp0) == 0) | (fRec2[1] >= 9.99999997e-07f));
			iRec3[0] = 1103515245 * iRec3[1] + 12345;
			float fTemp1 = fSlow11 * fRec2[0] * float(iRec3[0]);
			fVec0[0] = fTemp1;
			float fTemp2 = fTemp1 - fVec0[1];
			fRec0[0] = fSlow6 * (fSlow9 * fRec0[1] - fRec0[2]) + fSlow10 * fTemp2;
			iRec4[0] = iSlow13 & (iRec4[1] | (fRec5[1] >= 1.0f));
			int iTemp3 = fRec5[1] > 0.0f;
			fRec5[0] = (float(((iRec4[1] == 0) & iSlow13) & (fRec5[1] < 1.0f)) + fRec5[1] * (1.0f - fSlow18 * float(iRec4[1] & iTemp3))) * float(((iSlow16 & iTemp3) == 0) | (fRec5[1] >= 9.99999997e-07f));
			fRec6[0] = fSlow21 * (fSlow24 * fRec6[1] - fRec6[2]) + fSlow25 * fTemp2;
			iRec7[0] = iSlow13 & (iRec7[1] | (fRec8[1] >= 1.0f));
			int iTemp4 = fRec8[1] > 0.0f;
			fRec8[0] = (float(((iRec7[1] == 0) & iSlow13) & (fRec8[1] < 1.0f)) + fRec8[1] * (1.0f - fSlow27 * float(iRec7[1] & iTemp4))) * float(((iSlow16 & iTemp4) == 0) | (fRec8[1] >= 9.99999997e-07f));
			fRec9[0] = fSlow30 * (fSlow33 * fRec9[1] - fRec9[2]) + fSlow34 * fTemp2;
			iRec10[0] = iSlow13 & (iRec10[1] | (fRec11[1] >= 1.0f));
			int iTemp5 = fRec11[1] > 0.0f;
			fRec11[0] = (float(((iRec10[1] == 0) & iSlow13) & (fRec11[1] < 1.0f)) + fRec11[1] * (1.0f - fSlow36 * float(iRec10[1] & iTemp5))) * float(((iSlow16 & iTemp5) == 0) | (fRec11[1] >= 9.99999997e-07f));
			fRec12[0] = fSlow39 * (fSlow42 * fRec12[1] - fRec12[2]) + fSlow43 * fTemp2;
			iRec13[0] = iSlow13 & (iRec13[1] | (fRec14[1] >= 1.0f));
			int iTemp6 = fRec14[1] > 0.0f;
			fRec14[0] = (float(((iRec13[1] == 0) & iSlow13) & (fRec14[1] < 1.0f)) + fRec14[1] * (1.0f - fSlow45 * float(iRec13[1] & iTemp6))) * float(((iSlow16 & iTemp6) == 0) | (fRec14[1] >= 9.99999997e-07f));
			fRec15[0] = fSlow48 * (fSlow51 * fRec15[1] - fRec15[2]) + fSlow52 * fTemp2;
			iRec16[0] = iSlow13 & (iRec16[1] | (fRec17[1] >= 1.0f));
			int iTemp7 = fRec17[1] > 0.0f;
			fRec17[0] = (float(((iRec16[1] == 0) & iSlow13) & (fRec17[1] < 1.0f)) + fRec17[1] * (1.0f - fSlow54 * float(iRec16[1] & iTemp7))) * float(((iSlow16 & iTemp7) == 0) | (fRec17[1] >= 9.99999997e-07f));
			float fTemp8 = fSlow5 * fRec0[0] * fRec5[0] + fSlow20 * fRec6[0] * fRec8[0] + fSlow29 * fRec9[0] * fRec11[0] + fSlow38 * fRec12[0] * fRec14[0] + fSlow47 * fRec15[0] * fRec17[0];
			output0[i0] = FAUSTFLOAT(fSlow3 * fTemp8);
			output1[i0] = FAUSTFLOAT(fSlow55 * fTemp8);
			iRec1[1] = iRec1[0];
			fRec2[1] = fRec2[0];
			iRec3[1] = iRec3[0];
			fVec0[1] = fVec0[0];
			fRec0[2] = fRec0[1];
			fRec0[1] = fRec0[0];
			iRec4[1] = iRec4[0];
			fRec5[1] = fRec5[0];
			fRec6[2] = fRec6[1];
			fRec6[1] = fRec6[0];
			iRec7[1] = iRec7[0];
			fRec8[1] = fRec8[0];
			fRec9[2] = fRec9[1];
			fRec9[1] = fRec9[0];
			iRec10[1] = iRec10[0];
			fRec11[1] = fRec11[0];
			fRec12[2] = fRec12[1];
			fRec12[1] = fRec12[0];
			iRec13[1] = iRec13[0];
			fRec14[1] = fRec14[0];
			fRec15[2] = fRec15[1];
			fRec15[1] = fRec15[0];
			iRec16[1] = iRec16[0];
			fRec17[1] = fRec17[0];
		}
	}

};

#include <stdio.h>
#include <string>
#include "m_pd.h"

#define sym(name) xsym(name)
#define xsym(name) #name
#define faust_setup(name) xfaust_setup(name)
#define xfaust_setup(name) name ## _tilde_setup(void)

// time for "active" toggle xfades in secs
#define XFADE_TIME 0.1f

static t_class *faust_class;

struct t_faust {
  t_object x_obj;
#ifdef __MINGW32__
  /* This seems to be necessary as some as yet undetermined Pd routine seems
     to write past the end of x_obj on Windows. */
  int fence; /* dummy field (not used) */
#endif
  mydsp *dsp;
  PdUI *ui;
  std::string *label;
  int active, xfade, n_xfade, rate, n_in, n_out;
  t_sample **inputs, **outputs, **buf;
  t_outlet *out;
  t_sample f;
};

static t_symbol *s_button, *s_checkbox, *s_vslider, *s_hslider, *s_nentry,
  *s_vbargraph, *s_hbargraph;

static inline void zero_samples(int k, int n, t_sample **out)
{
  for (int i = 0; i < k; i++)
#ifdef __STDC_IEC_559__
    /* IEC 559 a.k.a. IEEE 754 floats can be initialized faster like this */
    memset(out[i], 0, n*sizeof(t_sample));
#else
    for (int j = 0; j < n; j++)
      out[i][j] = 0.0f;
#endif
}

static inline void copy_samples(int k, int n, t_sample **out, t_sample **in)
{
  for (int i = 0; i < k; i++)
    memcpy(out[i], in[i], n*sizeof(t_sample));
}

static t_int *faust_perform(t_int *w)
{
  t_faust *x = (t_faust *)(w[1]);
  int n = (int)(w[2]);
  if (!x->dsp || !x->buf) return (w+3);
  AVOIDDENORMALS;
  if (x->xfade > 0) {
    float d = 1.0f/x->n_xfade, f = (x->xfade--)*d;
    d = d/n;
    x->dsp->compute(n, x->inputs, x->buf);
    if (x->active)
      if (x->n_in == x->n_out)
	/* xfade inputs -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->inputs[i][j]+(1.0f-f)*x->buf[i][j];
      else
	/* xfade 0 -> buf */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = (1.0f-f)*x->buf[i][j];
    else
      if (x->n_in == x->n_out)
	/* xfade buf -> inputs */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j]+(1.0f-f)*x->inputs[i][j];
      else
	/* xfade buf -> 0 */
	for (int j = 0; j < n; j++, f -= d)
	  for (int i = 0; i < x->n_out; i++)
	    x->outputs[i][j] = f*x->buf[i][j];
  } else if (x->active) {
    x->dsp->compute(n, x->inputs, x->buf);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else if (x->n_in == x->n_out) {
    copy_samples(x->n_out, n, x->buf, x->inputs);
    copy_samples(x->n_out, n, x->outputs, x->buf);
  } else
    zero_samples(x->n_out, n, x->outputs);
  return (w+3);
}

static void faust_dsp(t_faust *x, t_signal **sp)
{
  int n = sp[0]->s_n, sr = (int)sp[0]->s_sr;
  if (x->rate <= 0) {
    /* default sample rate is whatever Pd tells us */
    PdUI *ui = x->ui;
    float *z = NULL;
    if (ui->nelems > 0 &&
	(z = (float*)malloc(ui->nelems*sizeof(float)))) {
      /* save the current control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  z[i] = *ui->elems[i].zone;
    }
    /* set the proper sample rate; this requires reinitializing the dsp */
    x->rate = sr;
    x->dsp->init(sr);
    if (z) {
      /* restore previous control values */
      for (int i = 0; i < ui->nelems; i++)
	if (ui->elems[i].zone)
	  *ui->elems[i].zone = z[i];
      free(z);
    }
  }
  if (n > 0)
    x->n_xfade = (int)(x->rate*XFADE_TIME/n);
  dsp_add(faust_perform, 2, x, n);
  for (int i = 0; i < x->n_in; i++)
    x->inputs[i] = sp[i+1]->s_vec;
  for (int i = 0; i < x->n_out; i++)
    x->outputs[i] = sp[x->n_in+i+1]->s_vec;
  if (x->buf != NULL)
    for (int i = 0; i < x->n_out; i++) {
      x->buf[i] = (t_sample*)malloc(n*sizeof(t_sample));
      if (x->buf[i] == NULL) {
	for (int j = 0; j < i; j++)
	  free(x->buf[j]);
	free(x->buf);
	x->buf = NULL;
	break;
      }
    }
}

static int pathcmp(const char *s, const char *t)
{
  int n = strlen(s), m = strlen(t);
  if (n == 0 || m == 0)
    return 0;
  else if (t[0] == '/')
    return strcmp(s, t);
  else if (n <= m || s[n-m-1] != '/')
    return strcmp(s+1, t);
  else
    return strcmp(s+n-m, t);
}

static void faust_any(t_faust *x, t_symbol *s, int argc, t_atom *argv)
{
  if (!x->dsp) return;
  PdUI *ui = x->ui;
  if (s == &s_bang) {
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label && ui->elems[i].zone) {
	t_atom args[6];
	t_symbol *_s;
	switch (ui->elems[i].type) {
	case UI_BUTTON:
	  _s = s_button;
	  break;
	case UI_CHECK_BUTTON:
	  _s = s_checkbox;
	  break;
	case UI_V_SLIDER:
	  _s = s_vslider;
	  break;
	case UI_H_SLIDER:
	  _s = s_hslider;
	  break;
	case UI_NUM_ENTRY:
	  _s = s_nentry;
	  break;
	case UI_V_BARGRAPH:
	  _s = s_vbargraph;
	  break;
	case UI_H_BARGRAPH:
	  _s = s_hbargraph;
	  break;
	default:
	  continue;
	}
	SETSYMBOL(&args[0], gensym(ui->elems[i].label));
	SETFLOAT(&args[1], *ui->elems[i].zone);
	SETFLOAT(&args[2], ui->elems[i].init);
	SETFLOAT(&args[3], ui->elems[i].min);
	SETFLOAT(&args[4], ui->elems[i].max);
	SETFLOAT(&args[5], ui->elems[i].step);
	outlet_anything(x->out, _s, 6, args);
      }
  } else {
    const char *label = s->s_name;
    int count = 0;
    for (int i = 0; i < ui->nelems; i++)
      if (ui->elems[i].label &&
	  pathcmp(ui->elems[i].label, label) == 0) {
	if (argc == 0) {
	  if (ui->elems[i].zone) {
	    t_atom arg;
	    SETFLOAT(&arg, *ui->elems[i].zone);
	    outlet_anything(x->out, gensym(ui->elems[i].label), 1, &arg);
	  }
	  ++count;
	} else if (argc == 1 &&
		   (argv[0].a_type == A_FLOAT ||
		    argv[0].a_type == A_DEFFLOAT) &&
		   ui->elems[i].zone) {
	  float f = atom_getfloat(argv);
	  *ui->elems[i].zone = f;
	  ++count;
	} else
	  pd_error(x, "[faust] %s: bad control argument: %s",
		   x->label->c_str(), label);
      }
    if (count == 0 && strcmp(label, "active") == 0) {
      if (argc == 0) {
	t_atom arg;
	SETFLOAT(&arg, (float)x->active);
	outlet_anything(x->out, gensym((char*)"active"), 1, &arg);
      } else if (argc == 1 &&
		 (argv[0].a_type == A_FLOAT ||
		  argv[0].a_type == A_DEFFLOAT)) {
	float f = atom_getfloat(argv);
	x->active = (int)f;
	x->xfade = x->n_xfade;
      }
    }
  }
}

static void faust_free(t_faust *x)
{
  if (x->label) delete x->label;
  if (x->dsp) delete x->dsp;
  if (x->ui) delete x->ui;
  if (x->inputs) free(x->inputs);
  if (x->outputs) free(x->outputs);
  if (x->buf) {
    for (int i = 0; i < x->n_out; i++)
      if (x->buf[i]) free(x->buf[i]);
    free(x->buf);
  }
}

static void *faust_new(t_symbol *s, int argc, t_atom *argv)
{
  t_faust *x = (t_faust*)pd_new(faust_class);
  int sr = -1;
  t_symbol *id = NULL;
  x->active = 1;
  for (int i = 0; i < argc; i++)
    if (argv[i].a_type == A_FLOAT || argv[i].a_type == A_DEFFLOAT)
      sr = (int)argv[i].a_w.w_float;
    else if (argv[i].a_type == A_SYMBOL || argv[i].a_type == A_DEFSYMBOL)
      id = argv[i].a_w.w_symbol;
  x->rate = sr;
  if (sr <= 0) sr = 44100;
  x->xfade = 0; x->n_xfade = (int)(sr*XFADE_TIME/64);
  x->inputs = x->outputs = x->buf = NULL;
    x->label = new std::string(sym(mydsp) "~");
  x->dsp = new mydsp();
  x->ui = new PdUI(sym(mydsp), id?id->s_name:NULL);
  if (!x->dsp || !x->ui || !x->label) goto error;
  if (id) {
    *x->label += " ";
    *x->label += id->s_name;
  }
  x->n_in = x->dsp->getNumInputs();
  x->n_out = x->dsp->getNumOutputs();
  if (x->n_in > 0)
    x->inputs = (t_sample**)malloc(x->n_in*sizeof(t_sample*));
  if (x->n_out > 0) {
    x->outputs = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
    x->buf = (t_sample**)malloc(x->n_out*sizeof(t_sample*));
  }
  if ((x->n_in > 0 && x->inputs == NULL) ||
      (x->n_out > 0 && (x->outputs == NULL || x->buf == NULL)))
    goto error;
  for (int i = 0; i < x->n_out; i++)
    x->buf[i] = NULL;
  x->dsp->init(sr);
  x->dsp->buildUserInterface(x->ui);
  for (int i = 0; i < x->n_in; i++)
    inlet_new(&x->x_obj, &x->x_obj.ob_pd, &s_signal, &s_signal);
  x->out = outlet_new(&x->x_obj, 0);
  for (int i = 0; i < x->n_out; i++)
    outlet_new(&x->x_obj, &s_signal);
  return (void *)x;
 error:
  faust_free(x);
  x->dsp = NULL; x->ui = NULL;
  x->inputs = x->outputs = x->buf = NULL;
  return (void *)x;
}

extern "C" void faust_setup(mydsp)
{
  t_symbol *s = gensym(sym(mydsp) "~");
  faust_class =
    class_new(s, (t_newmethod)faust_new, (t_method)faust_free,
	      sizeof(t_faust), CLASS_DEFAULT,
	      A_GIMME, A_NULL);
  class_addmethod(faust_class, (t_method)faust_dsp, gensym((char*)"dsp"), A_NULL);
  class_addanything(faust_class, faust_any);
  class_addmethod(faust_class, nullfn, &s_signal, A_NULL);
  s_button = gensym((char*)"button");
  s_checkbox = gensym((char*)"checkbox");
  s_vslider = gensym((char*)"vslider");
  s_hslider = gensym((char*)"hslider");
  s_nentry = gensym((char*)"nentry");
  s_vbargraph = gensym((char*)"vbargraph");
  s_hbargraph = gensym((char*)"hbargraph");
  /* give some indication that we're loaded and ready to go */
  mydsp dsp = mydsp();
  post("[faust] %s: %d inputs, %d outputs", sym(mydsp) "~",
       dsp.getNumInputs(), dsp.getNumOutputs());
}

#endif
